@@ -26,9 +26,17 @@
 #include "document.h"
 #include "theme.h"
 #include "settings.h"
+#include <gcu/application.h>
+#ifdef HAVE_GO_CONF_SYNC
+#	include <goffice/goffice.h>
+#endif
 #include <glib/gi18n-lib.h>
 #include <sys/stat.h>
 #include <cmath>
+#include <cstring>
+
+using namespace gcu;
+using namespace std;
 
 namespace gcp {
 
@@ -115,14 +123,30 @@ Theme::~Theme ()
 		g_free (m_FontFamily);
 	if (m_TextFontFamily)
 		g_free (m_TextFontFamily);
+	// set all remaining document client theme to NULL
+	// this is just a work around, but a ThemeClient class should be implemented
+	Document *doc;
+	std::set <gcu::Object*>::iterator i, end = m_Clients.end ();
+	for (i = m_Clients.begin (); i != end; i++) {
+		doc = dynamic_cast <Document *> (*i);
+		if (doc)
+			doc->SetTheme (NULL);
+	}
 }
 
 ThemeManager TheThemeManager;
 
+#ifdef HAVE_GO_CONF_SYNC
+static void on_config_changed (GOConfNode *node, gchar const *key, ThemeManager *manager)
+{
+	manager->OnConfigChanged (node, key);
+}
+#else
 static void on_config_changed (GConfClient *client, guint cnxn_id, GConfEntry *entry, ThemeManager *manager)
 {
 	manager->OnConfigChanged (client, cnxn_id, entry);
 }
+#endif
 
 // transform functions for gconf key values
 
@@ -148,6 +172,12 @@ static PangoWeight set_fontweight (int val)
 		case 7: return PANGO_WEIGHT_BOLD;
 		case 8: return PANGO_WEIGHT_ULTRABOLD;
 		case 9: return PANGO_WEIGHT_HEAVY;
+#if PANGO_VERSION_MAJOR > 1 || PANGO_VERSION_MINOR >= 24
+		case 1: return PANGO_WEIGHT_THIN;
+		case 38: return PANGO_WEIGHT_BOOK;
+		case 5: return PANGO_WEIGHT_MEDIUM;
+		case 10: return PANGO_WEIGHT_ULTRAHEAVY;
+#endif
 		default: return PANGO_WEIGHT_NORMAL;
 	}
 }
@@ -182,49 +212,57 @@ static int set_fontsize (double val) {return (int) (val * PANGO_SCALE);}
 ThemeManager::ThemeManager ()
 {
 	bindtextdomain(GETTEXT_PACKAGE, LOCALEDIR);
-#ifdef ENABLE_NLS
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-#endif
+	g_type_init ();
+#ifdef HAVE_GO_CONF_SYNC
+	libgoffice_init ();
+	m_ConfNode = go_conf_get_node (gcu::Application::GetConfDir (), GCP_CONF_DIR_SETTINGS);
+#else
 	GError *error = NULL;
 	m_ConfClient = gconf_client_get_default ();
-	gconf_client_add_dir (m_ConfClient, "/apps/gchempaint/settings", GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
-	GCU_GCONF_GET (ROOTDIR"bond-length", float, DefaultBondLength, 140.)
-	GCU_GCONF_GET (ROOTDIR"bond-angle", float, DefaultBondAngle, 120.)
-	GCU_GCONF_GET (ROOTDIR"bond-dist", float, DefaultBondDist, 5.)
-	GCU_GCONF_GET (ROOTDIR"bond-width", float, DefaultBondWidth, 1.0)
-	GCU_GCONF_GET (ROOTDIR"arrow-length", float, DefaultArrowLength, 200)
-	GCU_GCONF_GET (ROOTDIR"arrow-headA", float, DefaultArrowHeadA, 6.0)
-	GCU_GCONF_GET (ROOTDIR"arrow-headB", float, DefaultArrowHeadB, 8.0)
-	GCU_GCONF_GET (ROOTDIR"arrow-headC", float, DefaultArrowHeadC, 4.0)
-	GCU_GCONF_GET (ROOTDIR"arrow-dist", float, DefaultArrowDist, 5.0)
-	GCU_GCONF_GET (ROOTDIR"arrow-width", float, DefaultArrowWidth, 1.0)
-	GCU_GCONF_GET (ROOTDIR"hash-width", float, DefaultHashWidth, 1.0)
-	GCU_GCONF_GET (ROOTDIR"hash-dist", float, DefaultHashDist, 2.0)
-	GCU_GCONF_GET (ROOTDIR"stereo-width", float, DefaultStereoBondWidth, 5.0)
-	GCU_GCONF_GET_N_TRANSFORM (ROOTDIR"scale", float, DefaultZoomFactor, 0.25, inv)
-	GCU_GCONF_GET (ROOTDIR"padding", float, DefaultPadding, 2.0)
-	GCU_GCONF_GET (ROOTDIR"arrow-padding", float, DefaultArrowPadding, 16.0)
-	GCU_GCONF_GET (ROOTDIR"arrow-object-padding", float, DefaultArrowObjectPadding, 16.0)
-	GCU_GCONF_GET (ROOTDIR"stoichiometry-padding", float, DefaultStoichiometryPadding, 1.)
-	GCU_GCONF_GET (ROOTDIR"object-padding", float, DefaultObjectPadding, 16.0)
-	GCU_GCONF_GET (ROOTDIR"sign-padding", float, DefaultSignPadding, 8.0)
-	GCU_GCONF_GET (ROOTDIR"charge-sign-size", float, DefaultChargeSignSize, 9.)
-	GCU_GCONF_GET_STRING (ROOTDIR"font-family", DefaultFontFamily, "Bitstream Vera Sans")
-	GCU_GCONF_GET_N_TRANSFORM (ROOTDIR"font-style", int, DefaultFontStyle, 0, set_fontstyle)
-	GCU_GCONF_GET_N_TRANSFORM (ROOTDIR"font-weight", int, DefaultFontWeight, 4, set_fontweight)
-	GCU_GCONF_GET_N_TRANSFORM (ROOTDIR"font-variant", int, DefaultFontVariant, 0, set_fontvariant)
-	GCU_GCONF_GET_N_TRANSFORM (ROOTDIR"font-stretch", int, DefaultFontStretch, 4, set_fontstretch)
-	GCU_GCONF_GET_N_TRANSFORM (ROOTDIR"font-size", float, DefaultFontSize, 12., set_fontsize)
-	GCU_GCONF_GET_STRING (ROOTDIR"text-font-family", DefaultTextFontFamily, "Bitstream Vera Serif")
-	GCU_GCONF_GET_N_TRANSFORM (ROOTDIR"text-font-style", int, DefaultTextFontStyle, 0, set_fontstyle)
-	GCU_GCONF_GET_N_TRANSFORM (ROOTDIR"text-font-weight", int, DefaultTextFontWeight, 4, set_fontweight)
-	GCU_GCONF_GET_N_TRANSFORM (ROOTDIR"text-font-variant", int, DefaultTextFontVariant, 0, set_fontvariant)
-	GCU_GCONF_GET_N_TRANSFORM (ROOTDIR"text-font-stretch", int, DefaultTextFontStretch, 4, set_fontstretch)
-	GCU_GCONF_GET_N_TRANSFORM (ROOTDIR"text-font-size", float, DefaultTextFontSize, 12., set_fontsize)
-	m_NotificationId = gconf_client_notify_add (m_ConfClient, "/apps/gchempaint/settings", (GConfClientNotifyFunc) on_config_changed, this, NULL, NULL);
+	gconf_client_add_dir (m_ConfClient, "/apps/gchemutils/paint/settings", GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
+#endif
+	GCU_GCONF_GET ("bond-length", float, DefaultBondLength, 140.)
+	GCU_GCONF_GET ("bond-angle", float, DefaultBondAngle, 120.)
+	GCU_GCONF_GET ("bond-dist", float, DefaultBondDist, 5.)
+	GCU_GCONF_GET ("bond-width", float, DefaultBondWidth, 1.0)
+	GCU_GCONF_GET ("arrow-length", float, DefaultArrowLength, 200)
+	GCU_GCONF_GET ("arrow-headA", float, DefaultArrowHeadA, 6.0)
+	GCU_GCONF_GET ("arrow-headB", float, DefaultArrowHeadB, 8.0)
+	GCU_GCONF_GET ("arrow-headC", float, DefaultArrowHeadC, 4.0)
+	GCU_GCONF_GET ("arrow-dist", float, DefaultArrowDist, 5.0)
+	GCU_GCONF_GET ("arrow-width", float, DefaultArrowWidth, 1.0)
+	GCU_GCONF_GET ("hash-width", float, DefaultHashWidth, 1.0)
+	GCU_GCONF_GET ("hash-dist", float, DefaultHashDist, 2.0)
+	GCU_GCONF_GET ("stereo-width", float, DefaultStereoBondWidth, 5.0)
+	GCU_GCONF_GET_N_TRANSFORM ("scale", float, DefaultZoomFactor, 4., inv)
+	GCU_GCONF_GET ("padding", float, DefaultPadding, 2.0)
+	GCU_GCONF_GET ("arrow-padding", float, DefaultArrowPadding, 16.0)
+	GCU_GCONF_GET ("arrow-object-padding", float, DefaultArrowObjectPadding, 16.0)
+	GCU_GCONF_GET ("stoichiometry-padding", float, DefaultStoichiometryPadding, 1.)
+	GCU_GCONF_GET ("object-padding", float, DefaultObjectPadding, 16.0)
+	GCU_GCONF_GET ("sign-padding", float, DefaultSignPadding, 8.0)
+	GCU_GCONF_GET ("charge-sign-size", float, DefaultChargeSignSize, 9.)
+	GCU_GCONF_GET_STRING ("font-family", DefaultFontFamily, "Bitstream Vera Sans")
+	GCU_GCONF_GET_N_TRANSFORM ("font-style", int, DefaultFontStyle, 0, set_fontstyle)
+	GCU_GCONF_GET_N_TRANSFORM ("font-weight", int, DefaultFontWeight, 4, set_fontweight)
+	GCU_GCONF_GET_N_TRANSFORM ("font-variant", int, DefaultFontVariant, 0, set_fontvariant)
+	GCU_GCONF_GET_N_TRANSFORM ("font-stretch", int, DefaultFontStretch, 4, set_fontstretch)
+	GCU_GCONF_GET_N_TRANSFORM ("font-size", float, DefaultFontSize, 12., set_fontsize)
+	GCU_GCONF_GET_STRING ("text-font-family", DefaultTextFontFamily, "Bitstream Vera Serif")
+	GCU_GCONF_GET_N_TRANSFORM ("text-font-style", int, DefaultTextFontStyle, 0, set_fontstyle)
+	GCU_GCONF_GET_N_TRANSFORM ("text-font-weight", int, DefaultTextFontWeight, 4, set_fontweight)
+	GCU_GCONF_GET_N_TRANSFORM ("text-font-variant", int, DefaultTextFontVariant, 0, set_fontvariant)
+	GCU_GCONF_GET_N_TRANSFORM ("text-font-stretch", int, DefaultTextFontStretch, 4, set_fontstretch)
+	GCU_GCONF_GET_N_TRANSFORM ("text-font-size", float, DefaultTextFontSize, 12., set_fontsize)
 	// Build default theme from settings
-	m_Themes["Default"] = m_Themes[_("Default")] = new Theme ("Default");
-	m_Names.push_front (_("Default"));
+#ifdef HAVE_GO_CONF_SYNC
+	m_NotificationId = go_conf_add_monitor (m_ConfNode, NULL, (GOConfMonitorFunc) on_config_changed, this);
+#else
+	m_NotificationId = gconf_client_notify_add (m_ConfClient, "/apps/gchemutils/paint/settings", (GConfClientNotifyFunc) on_config_changed, this, NULL, NULL);
+#endif
+	m_Themes["GChemPaint"] = new Theme ("GChemPaint");
+	m_Names.push_front ("GChemPaint");
 	// load global themes
 	string path = PKGDATADIR;
 	path += "/paint/themes";
@@ -235,14 +273,16 @@ ThemeManager::ThemeManager ()
 		path = szhome;
 	path += "/.gchempaint/themes";
 	ParseDir (path, LOCAL_THEME_TYPE);
+	gchar *default_theme  =NULL;
+	GCU_GCONF_GET_STRING ("default-theme", default_theme, "GChemPaint");
+	m_DefaultTheme = m_Themes[default_theme];
+	g_free (default_theme);
+	if (!m_DefaultTheme)
+		m_DefaultTheme = m_Themes["GChemPaint"];
 }
 
 ThemeManager::~ThemeManager ()
 {
-	g_type_init ();
-	gconf_client_notify_remove (m_ConfClient, m_NotificationId);
-	gconf_client_remove_dir (m_ConfClient, "/apps/gchempaint/settings", NULL);
-	g_object_unref (m_ConfClient);
 	// save themes if needed, then delete
 	Theme *theme, *def = NULL;
 	map <string, Theme*>::iterator i, iend = m_Themes.end ();
@@ -282,13 +322,31 @@ ThemeManager::~ThemeManager ()
 	g_free (DefaultTextFontFamily);
 }
 
+void ThemeManager::Shutdown ()
+{
+#ifdef HAVE_GO_CONF_SYNC
+	go_conf_remove_monitor (m_NotificationId);
+	go_conf_free_node (m_ConfNode);
+	m_ConfNode = NULL;
+#else
+	gconf_client_notify_remove (m_ConfClient, m_NotificationId);
+	gconf_client_remove_dir (m_ConfClient, "/apps/gchemutils/paint/settings", NULL);
+	g_object_unref (m_ConfClient);
+	m_ConfClient = NULL;
+#endif
+}
+
 Theme *ThemeManager::GetTheme (char const *name)
 {
+	if (!strcmp (_(name), _("Default")))
+		return m_DefaultTheme;
 	return m_Themes[name];
 }
 
 Theme *ThemeManager::GetTheme (string &name)
 {
+	if (name == "Default" || name == _("Default") )
+		return m_DefaultTheme;
 	return m_Themes[name.c_str ()];
 }
 
@@ -297,120 +355,98 @@ list <string> const &ThemeManager::GetThemesNames ()
 	return m_Names;
 }
 
+#ifdef HAVE_GO_CONF_SYNC
+void ThemeManager::OnConfigChanged (GOConfNode *node, gchar const *name)
+{
+#else
 void ThemeManager::OnConfigChanged (GConfClient *client, guint cnxn_id, GConfEntry *entry)
 {
 	if (client != m_ConfClient)
 		return;	// we might want an error message?
 	if (cnxn_id != m_NotificationId)
 		return;	// we might want an error message?
-	Theme *Theme = m_Themes["Default"];
-	if (!strcmp (gconf_entry_get_key (entry),ROOTDIR"bond-length")) {
-		DefaultBondLength = gconf_value_get_float (gconf_entry_get_value (entry));
-		Theme->m_BondLength = DefaultBondLength;
-	} else if (!strcmp (gconf_entry_get_key (entry),ROOTDIR"bond-angle"))  {
-		DefaultBondAngle = gconf_value_get_float (gconf_entry_get_value (entry));
-		Theme->m_BondAngle = DefaultBondAngle;
-	} else if (!strcmp (gconf_entry_get_key (entry),ROOTDIR"bond-dist"))  {
-		DefaultBondDist = gconf_value_get_float (gconf_entry_get_value (entry));
-		Theme->m_BondDist = DefaultBondDist;
-	} else if (!strcmp (gconf_entry_get_key (entry),ROOTDIR"bond-width"))  {
-		DefaultBondWidth = gconf_value_get_float (gconf_entry_get_value (entry));
-		Theme->m_BondWidth = DefaultBondWidth;
-	} else if (!strcmp (gconf_entry_get_key (entry),ROOTDIR"arrow-length"))  {
-		DefaultArrowLength = gconf_value_get_float (gconf_entry_get_value (entry));
-		Theme->m_ArrowLength = DefaultArrowLength;
-	} else if (!strcmp (gconf_entry_get_key (entry),ROOTDIR"arrow-headA"))  {
-		DefaultArrowHeadA = gconf_value_get_float (gconf_entry_get_value (entry));
-		Theme->m_ArrowHeadA = DefaultArrowHeadA;
-	} else if (!strcmp (gconf_entry_get_key (entry),ROOTDIR"arrow-headB"))  {
-		DefaultArrowHeadB = gconf_value_get_float (gconf_entry_get_value (entry));
-		Theme->m_ArrowHeadB = DefaultArrowHeadB;
-	} else if (!strcmp (gconf_entry_get_key (entry),ROOTDIR"arrow-headC"))  {
-		DefaultArrowHeadC = gconf_value_get_float (gconf_entry_get_value (entry));
-		Theme->m_ArrowHeadC = DefaultArrowHeadC;
-	} else if (!strcmp (gconf_entry_get_key (entry),ROOTDIR"arrow-dist"))  {
-		DefaultArrowDist = gconf_value_get_float (gconf_entry_get_value (entry));
-		Theme->m_ArrowDist = DefaultArrowDist;
-	} else if (!strcmp (gconf_entry_get_key (entry),ROOTDIR"arrow-width"))  {
-		DefaultArrowWidth = gconf_value_get_float (gconf_entry_get_value (entry));
-		Theme->m_ArrowWidth = DefaultArrowWidth;
-	} else if (!strcmp (gconf_entry_get_key (entry),ROOTDIR"hash-width"))  {
-		DefaultHashWidth = gconf_value_get_float (gconf_entry_get_value (entry));
-		Theme->m_HashWidth = DefaultHashWidth;
-	} else if (!strcmp (gconf_entry_get_key (entry),ROOTDIR"hash-dist"))  {
-		DefaultHashDist = gconf_value_get_float (gconf_entry_get_value (entry));
-		Theme->m_HashDist = DefaultHashDist;
-	} else if (!strcmp (gconf_entry_get_key (entry),ROOTDIR"stereo-width"))  {
-		DefaultStereoBondWidth = gconf_value_get_float (gconf_entry_get_value (entry));
-		Theme->m_StereoBondWidth = DefaultStereoBondWidth;
-	} else if (!strcmp (gconf_entry_get_key (entry),ROOTDIR"scale"))  {
-		double scale = gconf_value_get_float (gconf_entry_get_value (entry));
-		if (scale > 1e-5) {
-			DefaultZoomFactor = 1 / scale;
-			Theme->m_ZoomFactor = DefaultZoomFactor;
+	char const *name;
+#endif
+	Theme *theme = m_Themes["GChemPaint"];
+	GCU_UPDATE_KEY ("bond-length", float, DefaultBondLength, theme->m_BondLength = DefaultBondLength;)
+	GCU_UPDATE_KEY ("bond-angle", float, DefaultBondAngle, theme->m_BondAngle = DefaultBondAngle;)
+	GCU_UPDATE_KEY ("bond-dist", float, DefaultBondDist, theme->m_BondDist = DefaultBondDist;)
+	GCU_UPDATE_KEY ("bond-width", float, DefaultBondWidth, theme->m_BondWidth = DefaultBondWidth;)
+	GCU_UPDATE_KEY ("arrow-length", float, DefaultArrowLength, theme->m_ArrowLength = DefaultArrowLength;)
+	GCU_UPDATE_KEY ("arrow-headA", float, DefaultArrowHeadA, theme->m_ArrowHeadA = DefaultArrowHeadA;)
+	GCU_UPDATE_KEY ("arrow-headB", float, DefaultArrowHeadB, theme->m_ArrowHeadB = DefaultArrowHeadB;)
+	GCU_UPDATE_KEY ("arrow-headA", float, DefaultArrowHeadC, theme->m_ArrowHeadC = DefaultArrowHeadC;)
+	GCU_UPDATE_KEY ("arrow-dist", float, DefaultArrowDist, theme->m_ArrowDist = DefaultArrowDist;)
+	GCU_UPDATE_KEY ("arrow-width", float, DefaultArrowWidth, theme->m_ArrowWidth = DefaultArrowWidth;)
+	GCU_UPDATE_KEY ("hash-width", float, DefaultHashWidth, theme->m_HashWidth = DefaultHashWidth;)
+	GCU_UPDATE_KEY ("hash-dist", float, DefaultHashDist, theme->m_HashDist = DefaultHashDist;)
+	GCU_UPDATE_KEY ("stereo-width", float, DefaultStereoBondWidth, theme->m_StereoBondWidth = DefaultStereoBondWidth;)
+	double x;
+	GCU_UPDATE_KEY ("scale", float, x,
+		if (x > 1e-5) {
+			DefaultZoomFactor = 1 / x;
+			theme->m_ZoomFactor = DefaultZoomFactor;
 		}
-	} else if (!strcmp (gconf_entry_get_key (entry),ROOTDIR"padding"))  {
-		DefaultPadding = gconf_value_get_float (gconf_entry_get_value (entry));
-		Theme->m_Padding = DefaultPadding;
-	} else if (!strcmp (gconf_entry_get_key (entry),ROOTDIR"stoichiometry-padding"))  {
-		DefaultStoichiometryPadding = gconf_value_get_float (gconf_entry_get_value (entry));
-		Theme->m_StoichiometryPadding = DefaultStoichiometryPadding;
-	} else if (!strcmp (gconf_entry_get_key (entry),ROOTDIR"object-padding"))  {
-		DefaultObjectPadding = gconf_value_get_float (gconf_entry_get_value (entry));
-		Theme->m_ObjectPadding = DefaultObjectPadding;
-	} else if (!strcmp (gconf_entry_get_key (entry),ROOTDIR"charge-sign-padding"))  {
-		DefaultSignPadding = gconf_value_get_float (gconf_entry_get_value (entry));
-		Theme->m_SignPadding = DefaultSignPadding;
-	} else if (!strcmp (gconf_entry_get_key (entry),ROOTDIR"charge-sign-size"))  {
-		DefaultChargeSignSize = gconf_value_get_float (gconf_entry_get_value (entry));
-		Theme->m_ChargeSignSize = DefaultChargeSignSize;
-	} else if (!strcmp (gconf_entry_get_key (entry),ROOTDIR"font-family"))  {
-		char const *name = gconf_value_get_string (gconf_entry_get_value (entry));
+	)
+	GCU_UPDATE_KEY ("padding", float, DefaultPadding, theme->m_Padding = DefaultPadding;)
+	GCU_UPDATE_KEY ("stoichiometry-padding", float, DefaultStoichiometryPadding, theme->m_StoichiometryPadding = DefaultStoichiometryPadding;)
+	GCU_UPDATE_KEY ("object-padding", float, DefaultObjectPadding, theme->m_ObjectPadding = DefaultObjectPadding;)
+	GCU_UPDATE_KEY ("charge-sign-padding", float, DefaultSignPadding, theme->m_SignPadding = DefaultSignPadding;)
+	GCU_UPDATE_KEY ("charge-sign-size", float, DefaultChargeSignSize, theme->m_ChargeSignSize = DefaultChargeSignSize;)
+	GCU_UPDATE_KEY ("font-family", string, name,
 		if (name) {
 			if (DefaultFontFamily != NULL)
 				g_free (DefaultFontFamily);
 			DefaultFontFamily = g_strdup (name);
-			Theme->m_FontFamily = DefaultFontFamily;
-		}
-	} else if (!strcmp (gconf_entry_get_key (entry),ROOTDIR"font-style"))  {
-		DefaultFontStyle = set_fontstyle (gconf_value_get_int (gconf_entry_get_value (entry)));
-		Theme->m_FontStyle = DefaultFontStyle;
-	} else if (!strcmp (gconf_entry_get_key (entry),ROOTDIR"font-weight"))  {
-		DefaultFontWeight = set_fontweight (gconf_value_get_int (gconf_entry_get_value (entry)));
-		Theme->m_FontWeight = DefaultFontWeight;
-	} else if (!strcmp (gconf_entry_get_key (entry),ROOTDIR"font-variant"))  {
-		DefaultFontVariant = set_fontvariant (gconf_value_get_int (gconf_entry_get_value (entry)));
-		Theme->m_FontVariant = DefaultFontVariant;
-	} else if (!strcmp (gconf_entry_get_key (entry),ROOTDIR"font-stretch"))  {
-		DefaultFontStretch = set_fontstretch (gconf_value_get_int (gconf_entry_get_value (entry)));
-		Theme->m_FontStretch = DefaultFontStretch;
-	} else if (!strcmp (gconf_entry_get_key (entry),ROOTDIR"font-size"))  {
-		DefaultFontSize = set_fontsize (gconf_value_get_float (gconf_entry_get_value (entry)));
-		Theme->m_FontSize = DefaultFontSize;
-	} else if (!strcmp (gconf_entry_get_key (entry),ROOTDIR"text-font-family"))  {
-		char const *name = gconf_value_get_string (gconf_entry_get_value (entry));
+			theme->m_FontFamily = DefaultFontFamily;
+		})
+	int n;
+	GCU_UPDATE_KEY ("font-style", int, n, {
+		DefaultFontStyle = set_fontstyle (n);
+		theme->m_FontStyle = DefaultFontStyle;
+	})
+	GCU_UPDATE_KEY ("font-weight", int, n, {
+		DefaultFontWeight = set_fontweight (n);
+		theme->m_FontWeight = DefaultFontWeight;
+	})
+	GCU_UPDATE_KEY ("font-variant", int, n, {
+		DefaultFontVariant = set_fontvariant (n);
+		theme->m_FontVariant = DefaultFontVariant;
+	})
+	GCU_UPDATE_KEY ("font-stretch", int, n, {
+		DefaultFontStretch = set_fontstretch (n);
+		theme->m_FontStretch = DefaultFontStretch;
+	})
+	GCU_UPDATE_KEY ("font-size", float, DefaultFontSize, theme->m_FontSize = DefaultFontSize;)
+	GCU_UPDATE_KEY ("text-font-family", string, name,
 		if (name) {
 			if (DefaultTextFontFamily != NULL)
 				g_free (DefaultTextFontFamily);
 			DefaultTextFontFamily = g_strdup (name);
-			Theme->m_TextFontFamily = DefaultTextFontFamily;
-		}
-	} else if (!strcmp (gconf_entry_get_key (entry),ROOTDIR"text-font-style"))  {
-		DefaultTextFontStyle = set_fontstyle (gconf_value_get_int (gconf_entry_get_value (entry)));
-		Theme->m_TextFontStyle = DefaultTextFontStyle;
-	} else if (!strcmp (gconf_entry_get_key (entry),ROOTDIR"text-font-weight"))  {
-		DefaultTextFontWeight = set_fontweight (gconf_value_get_int (gconf_entry_get_value (entry)));
-		Theme->m_TextFontWeight = DefaultTextFontWeight;
-	} else if (!strcmp (gconf_entry_get_key (entry),ROOTDIR"text-font-variant"))  {
-		DefaultTextFontVariant = set_fontvariant (gconf_value_get_int (gconf_entry_get_value (entry)));
-		Theme->m_TextFontVariant = DefaultTextFontVariant;
-	} else if (!strcmp (gconf_entry_get_key (entry),ROOTDIR"text-font-stretch"))  {
-		DefaultTextFontStretch = set_fontstretch (gconf_value_get_int (gconf_entry_get_value (entry)));
-		Theme->m_TextFontStretch = DefaultTextFontStretch;
-	} else if (!strcmp (gconf_entry_get_key (entry),ROOTDIR"text-font-size"))  {
-		DefaultTextFontSize = set_fontsize (gconf_value_get_float (gconf_entry_get_value (entry)));
-		Theme->m_TextFontSize = DefaultTextFontSize;
-	}
+			theme->m_TextFontFamily = DefaultTextFontFamily;
+		})
+	GCU_UPDATE_KEY ("text-font-style", int, n, {
+		DefaultTextFontStyle = set_fontstyle (n);
+		theme->m_TextFontStyle = DefaultTextFontStyle;
+	})
+	GCU_UPDATE_KEY ("text-font-weight", int, n, {
+		DefaultTextFontWeight = set_fontweight (n);
+		theme->m_TextFontWeight = DefaultTextFontWeight;
+	})
+	GCU_UPDATE_KEY ("text-font-variant", int, n, {
+		DefaultTextFontVariant = set_fontvariant (n);
+		theme->m_TextFontVariant = DefaultTextFontVariant;
+	})
+	GCU_UPDATE_KEY ("text-font-stretch", int, n, {
+		DefaultTextFontStretch = set_fontstretch (n);
+		theme->m_TextFontStretch = DefaultTextFontStretch;
+	})
+	GCU_UPDATE_KEY ("text-font-size", float, DefaultTextFontSize, theme->m_TextFontSize = DefaultTextFontSize;)
+	GCU_UPDATE_KEY ("default-theme", string, name, {
+		theme = m_Themes[name];
+		if (theme)
+			m_DefaultTheme = theme;
+	})
 }
 
 Theme *ThemeManager::CreateNewTheme (Theme *theme)
@@ -477,6 +513,8 @@ void ThemeManager::ParseDir (string &path, ThemeType type)
 	if (dir) {
 		path += "/";
 		while ((name = g_dir_read_name (dir))) {
+			if (name[strlen (name) - 1] == '~')
+				continue; // don't read backups
 			filename = path + name;
 			doc = xmlParseFile (filename.c_str ());
 			node = doc->children;
@@ -494,6 +532,8 @@ void ThemeManager::ParseDir (string &path, ThemeType type)
 					if (theme->m_ThemeType == DEFAULT_THEME_TYPE || theme->m_ThemeType == GLOBAL_THEME_TYPE)
 						name = _(name);
 					if (m_Themes.find (name) != m_Themes.end ()) {
+						delete theme;
+						continue;
 					}
 					theme->m_ThemeType = type;
 					m_Themes[name] = theme;
@@ -509,9 +549,15 @@ void ThemeManager::ParseDir (string &path, ThemeType type)
 void ThemeManager::AddFileTheme (Theme *theme, char const *label)
 {
 	string name = theme->GetName ().c_str ();
+	if (name == "Default")
+		name = "GChemPaint";
+	string theme_name = name;
 	if (m_Themes.find (name) != m_Themes.end ()) {
-		name = string (label) + ":" + name;
+		name = string ((label)? label: _("Unknown")) + ":" + name;
 	}
+	int i = 0;
+	while (m_Themes.find (name) != m_Themes.end ())
+		name = string ((label)? label: _("Unknown")) + ((char) ('0' + i++)) + ":" + theme_name;
 	m_Themes[name] = theme;
 	m_Names.push_back (name);
 }
@@ -537,6 +583,14 @@ void ThemeManager::ChangeThemeName (Theme *theme, char const *name)
 	m_Themes[name] = theme;
 	m_Names.push_back (name);
 }
+ 
+void ThemeManager::SetDefaultTheme (char const *name)
+{
+	Theme *theme = m_Themes[name];
+	if (theme) {
+		m_DefaultTheme = theme;
+	}
+}
 
 bool Theme::Save (xmlDocPtr xml)
 {
@@ -644,6 +698,20 @@ bool Theme::Save (xmlDocPtr xml)
 	case PANGO_WEIGHT_HEAVY:
 		buf = const_cast<char*> ("heavy");
 		break;
+#if PANGO_VERSION_MAJOR > 1 || PANGO_VERSION_MINOR >= 24
+	case PANGO_WEIGHT_THIN:
+		buf = const_cast<char*> ("thin");
+		break;
+	case PANGO_WEIGHT_BOOK:
+		buf = const_cast<char*> ("book");
+		break;
+	case PANGO_WEIGHT_MEDIUM:
+		buf = const_cast<char*> ("medium");
+		break;
+	case PANGO_WEIGHT_ULTRAHEAVY:
+		buf = const_cast<char*> ("ultra-heavy");
+		break;
+#endif
 	}
 	if (buf)
 		xmlNewProp (node, (xmlChar*) "font-weight", (xmlChar*) buf);
@@ -721,6 +789,20 @@ bool Theme::Save (xmlDocPtr xml)
 	case PANGO_WEIGHT_HEAVY:
 		buf = const_cast<char*> ("heavy");
 		break;
+#if PANGO_VERSION_MAJOR > 1 || PANGO_VERSION_MINOR >= 24
+	case PANGO_WEIGHT_THIN:
+		buf = const_cast<char*> ("thin");
+		break;
+	case PANGO_WEIGHT_BOOK:
+		buf = const_cast<char*> ("book");
+		break;
+	case PANGO_WEIGHT_MEDIUM:
+		buf = const_cast<char*> ("medium");
+		break;
+	case PANGO_WEIGHT_ULTRAHEAVY:
+		buf = const_cast<char*> ("ultra-heavy");
+		break;
+#endif
 	}
 	if (buf)
 		xmlNewProp (node, (xmlChar*) "text-font-weight", (xmlChar*) buf);
@@ -837,6 +919,16 @@ bool Theme::Load (xmlNodePtr node)
 			m_FontWeight = PANGO_WEIGHT_ULTRABOLD;
 		else if (!strcmp (buf, "heavy"))
 			m_FontWeight = PANGO_WEIGHT_HEAVY;
+#if PANGO_VERSION_MAJOR > 1 || PANGO_VERSION_MINOR >= 24
+		else if (!strcmp (buf, "thin"))
+			m_FontWeight = PANGO_WEIGHT_THIN;
+		else if (!strcmp (buf, "book"))
+			m_FontWeight = PANGO_WEIGHT_BOOK;
+		else if (!strcmp (buf, "medium"))
+			m_FontWeight = PANGO_WEIGHT_MEDIUM;
+		else if (!strcmp (buf, "ultra-heavy"))
+			m_FontWeight = PANGO_WEIGHT_ULTRAHEAVY;
+#endif
 		xmlFree (buf);
 	}
 	buf = (char*) xmlGetProp (node, (xmlChar*) "font-variant");
@@ -903,6 +995,16 @@ bool Theme::Load (xmlNodePtr node)
 			m_TextFontWeight = PANGO_WEIGHT_ULTRABOLD;
 		else if (!strcmp (buf, "heavy"))
 			m_TextFontWeight = PANGO_WEIGHT_HEAVY;
+#if PANGO_VERSION_MAJOR > 1 || PANGO_VERSION_MINOR >= 24
+		else if (!strcmp (buf, "thin"))
+			m_TextFontWeight = PANGO_WEIGHT_THIN;
+		else if (!strcmp (buf, "book"))
+			m_TextFontWeight = PANGO_WEIGHT_BOOK;
+		else if (!strcmp (buf, "medium"))
+			m_TextFontWeight = PANGO_WEIGHT_MEDIUM;
+		else if (!strcmp (buf, "ultra-heavy"))
+			m_TextFontWeight = PANGO_WEIGHT_ULTRAHEAVY;
+#endif
 		xmlFree (buf);
 	}
 	buf = (char*) xmlGetProp (node, (xmlChar*) "text-font-variant");
