@@ -4,7 +4,7 @@
  * Gnome Chemistry Utils
  * programs/3d/window.cc 
  *
- * Copyright (C) 2006 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2006-2008 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -28,50 +28,51 @@
 #include "view.h"
 #include "window.h"
 #include <gcu/print-setup-dlg.h>
-#include <goffice/gtk/go-action-combo-color.h>
-#include <goffice/utils/go-image.h>
-#include <libgnomevfs/gnome-vfs.h>
-#include <gtk/gtk.h>
+#ifndef GOFFICE_HAS_GLOBAL_HEADER
+#   include <goffice/gtk/go-action-combo-color.h>
+#   include <goffice/utils/go-image.h>
+#   include <gtk/gtk.h>
+#endif
 #include <glib/gi18n.h>
 
 //Callbacks
-static bool on_delete_event (GtkWidget* widget, GdkEvent *event, gc3dWindow* Win)
+static bool on_delete_event (GtkWidget *widget, GdkEvent *event, gc3dWindow *Win)
 {
 	delete Win;
 	return false;
 }
 
-static void on_file_open (GtkWidget *widget, gc3dWindow* Win)
+static void on_file_open (GtkWidget *widget, gc3dWindow *Win)
 {
 	Win->OnFileOpen ();
 }
 
-static void on_file_save_as_image(GtkWidget* widget, gc3dWindow* Win)
+static void on_file_save_as_image(GtkWidget *widget, gc3dWindow *Win)
 {
 	Win->GetApp ()->OnSaveAsImage (Win->GetDoc ());
 }
 
-static void on_file_close (GtkWidget *widget, gc3dWindow* Win)
+static void on_file_close (GtkWidget *widget, gc3dWindow *Win)
 {
 	Win->OnFileClose ();
 }
 
-static void on_page_setup (GtkWidget *widget, gc3dWindow* Win)
+static void on_page_setup (GtkWidget *widget, gc3dWindow *Win)
 {
 	Win->OnPageSetup ();
 }
 
-static void on_print_preview (GtkWidget *widget, gc3dWindow* Win)
+static void on_print_preview (GtkWidget *widget, gc3dWindow *Win)
 {
 	Win->GetView ()->Print (true);
 }
 
-static void on_file_print (GtkWidget *widget, gc3dWindow* Win)
+static void on_file_print (GtkWidget *widget, gc3dWindow *Win)
 {
 	Win->GetView ()->Print (false);
 }
 
-static void on_quit (GtkWidget *widget, gc3dWindow* Win)
+static void on_quit (GtkWidget *widget, gc3dWindow *Win)
 {
 	gc3dApplication *App = Win->GetApp ();
 	App->OnQuit ();
@@ -94,35 +95,37 @@ static void on_display (GtkRadioAction *action, GtkRadioAction *current, gc3dWin
 	window->GetView ()->Update ();
 }
 
-static void on_help (GtkWidget *widget, gc3dWindow* window)
+static void on_help (GtkWidget *widget, gc3dWindow *window)
 {
 	window->GetApp ()->OnHelp ();
 }
 
-static void on_web (GtkWidget *widget, gc3dWindow* window)
+static void on_web (GtkWidget *widget, gc3dWindow *window)
 {
 	window->GetApp ()->OnWeb ();
 }
 
-static void on_mail (GtkWidget *widget, gc3dWindow* window)
+static void on_mail (GtkWidget *widget, gc3dWindow *window)
 {
 	window->GetApp ()->OnMail ();
 }
 
-static void on_bug (GtkWidget *widget, gc3dWindow* window)
+static void on_bug (GtkWidget *widget, gc3dWindow *window)
 {
 	window->GetApp ()->OnBug ();
 }
 
-static void on_about_activate_url (GtkAboutDialog *about, const gchar *url, gpointer data)
+static void on_live_assistance (GtkWidget *widget, gc3dWindow *window)
 {
-	GnomeVFSResult error = gnome_vfs_url_show(url);
-	if (error != GNOME_VFS_OK) {
-		g_print("GnomeVFSResult while trying to launch URL in about dialog: error %u\n", error);
-	}
+	window->GetApp ()->OnLiveAssistance ();
 }
 
-static void on_about (GtkWidget *widget, void *data)
+static void on_about_activate_url (GtkAboutDialog *about, const gchar *url, gpointer data)
+{
+	reinterpret_cast <gc3dWindow *> (data)->GetApp ()->OnWeb (url);
+}
+
+static void on_about (GtkWidget *widget, gc3dWindow *window)
 {
 	const gchar * authors[] = {"Jean Bréfort", NULL};
 	const gchar * comments = _("GChem3D is a molecular structures viewer for Gnome");
@@ -142,7 +145,7 @@ static void on_about (GtkWidget *widget, void *data)
 		"Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02111-1307\n"
 		"USA";
 	
-	gtk_about_dialog_set_url_hook(on_about_activate_url, NULL, NULL);
+	gtk_about_dialog_set_url_hook (on_about_activate_url, window, NULL);
 
 	/* Note to translators: replace the following string with the appropriate credits for you lang */
 	const gchar * translator_credits = _("translator_credits");
@@ -188,6 +191,8 @@ static GtkActionEntry entries[] = {
 		  N_("View help for the Molecules Viewer"), G_CALLBACK (on_help) },
 	  { "Web", NULL, N_("Gnome Chemistry Utils on the _web"), NULL,
 		  N_("Browse the Gnome Chemistry Utils's web site"), G_CALLBACK (on_web) },
+	  { "LiveAssistance", NULL, N_("Live assistance"), NULL,
+		  N_("Open the Gnome Chemistry Utils IRC channel"), G_CALLBACK (on_live_assistance) },
 	  { "Mail", NULL, N_("_Ask a question"), NULL,
 		  N_("Ask a question about the Gnome Chemistry Utils"), G_CALLBACK (on_mail) },
 	  { "Bug", NULL, N_("Report _Bugs"), NULL,
@@ -235,39 +240,15 @@ static const char *ui_description =
 "    </menu>"
 "    <menu action='HelpMenu'>"
 "      <menuitem action='Help'/>"
-"      <placeholder name='mail'/>"
-"      <placeholder name='web'/>"
-"      <placeholder name='bug'/>"
+"      <menuitem action='Mail'/>"
+"      <menuitem action='Web'/>"
+"      <menuitem action='LiveAssistance'/>"
+"      <menuitem action='Bug'/>"
 "      <menuitem action='About'/>"
 "    </menu>"
 "  </menubar>"
 "</ui>";
 
-static const char *ui_mail_description =
-"<ui>"
-"  <menubar name='MainMenu'>"
-"    <menu action='HelpMenu'>"
-"      <placeholder name='mail'>"
-"        <menuitem action='Mail'/>"
-"      </placeholder>"
-"    </menu>"
-"  </menubar>"
-"</ui>";
-
-static const char *ui_web_description =
-"<ui>"
-"  <menubar name='MainMenu'>"
-"    <menu action='HelpMenu'>"
-"      <placeholder name='web'>"
-"        <menuitem action='Web'/>"
-"      </placeholder>"
-"      <placeholder name='bug'>"
-"        <menuitem action='Bug'/>"
-"      </placeholder>"
-"    </menu>"
-"  </menubar>"
-"</ui>";
-
 gc3dWindow::gc3dWindow (gc3dApplication *App, gc3dDocument *Doc)
 {
 	m_App = App;
@@ -303,16 +284,9 @@ gc3dWindow::gc3dWindow (gc3dApplication *App, gc3dDocument *Doc)
 		g_error_free (error);
 		exit (EXIT_FAILURE);
 	}
-	if (App->HasWebBrowser () && !gtk_ui_manager_add_ui_from_string (ui_manager, ui_web_description, -1, &error)) {
-		g_message ("building menus failed: %s", error->message);
-		g_error_free (error);
-	}
-	if (App->HasMailAgent () && !gtk_ui_manager_add_ui_from_string (ui_manager, ui_mail_description, -1, &error)) {
-		g_message ("building menus failed: %s", error->message);
-		g_error_free (error);
-	}
 	GtkWidget *menu = gtk_ui_manager_get_widget (ui_manager, "/MainMenu/FileMenu/Open");
 	GtkWidget *w = gtk_recent_chooser_menu_new_for_manager (App->GetRecentManager ());
+	gtk_recent_chooser_set_sort_type (GTK_RECENT_CHOOSER (w), GTK_RECENT_SORT_MRU);
 	GtkRecentFilter *filter = gtk_recent_filter_new ();
 	gtk_recent_filter_add_mime_type (filter, "chemical/x-cml");
 	gtk_recent_filter_add_mime_type (filter, "chemical/x-mdl-molfile");
